// Package `tracking` defines a generic track (sort of record) type
// and a container type that allows storing of tracks in a SQL database.
package tracking

import (
	"encoding/json"
	"time"

	lib "git.sr.ht/~cco/go-scopes"
	"git.sr.ht/~cco/go-scopes/storage"
	sql "git.sr.ht/~cco/go-scopes/storage"
)

type Track interface {
	TrackId() lib.Ident
}

type BaseTrack = track
type BaseContainer = container

// basic track implementation

type track struct {
	trackId   lib.Ident
	head      lib.StrMap
	timeStamp *time.Time
	data      lib.Map
	container *container
}

func SimpleTrack(taskId, userName string, data lib.Map) track {
	return track{
		head: lib.StrMap{"taskId": taskId, "userName": userName},
		data: data,
	}
}

func (tr *track) TrackId() lib.Ident {
	return tr.trackId
}

// basic container implementation

type container struct {
	tableName  string
	headFields []string
	indexes    [][]string
	storage    *sql.Storage
}

func SimpleContainer(db *sql.Storage) *container {
	return &container{
		tableName:  "tracks",
		headFields: []string{"taskId", "userName"},
		indexes:    [][]string{[]string{"taskId", "userName"}, []string{"userName"}},
		storage:    db,
	}
}

func (cont *container) Insert(tr *track) lib.Ident {
	var columns []string
	var values []any
	for _, k := range cont.headFields {
		columns = append(columns, k)
		values = append(values, tr.head[k])
	}
	columns = append(columns, "Data")
	b, _ := json.Marshal(tr.data)
	values = append(values, b)
	db := cont.storage
	data := lib.Map{
		"schema":    db.Schema,
		"tablename": cont.tableName,
		"columns":   columns,
	}
	sql := storage.BuildSql(SqlInsert, data)
	if res := storage.QueryCol[uint](db, sql, values...); res != nil {
		trid := lib.Ident(res[0])
		tr.trackId = trid
		tr.container = cont
		return trid
	}
	return 0
}

func (cont *container) CreateTable() *container {
	db := cont.storage
	data := lib.Map{
		"schema":     db.Schema,
		"tablename":  cont.tableName,
		"headFields": cont.headFields,
		"indexes":    cont.indexes,
		"params":     db.Params,
	}
	sql := storage.BuildSql(SqlCreate, data)
	if _, err := db.Exec(sql); err == nil {
		return cont
	}
	return nil
}
